//
//  MessageDataSource.swift
//  SampleApp
//
//  Created by Federico Zanetello on 5/14/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import AmitySDK

final class MessageDataSource {
    private let messagesCollection: AmityCollection<AmityMessage>
    private var messagesToken: AmityNotificationToken?
    private let reverse: Bool
    private var lastMessageHash: Int = -1
    weak var dataSourceObserver: MessageDataSourceListener?
    var isFirstPageLoaded = false
    
    init(messagesCollection: AmityCollection<AmityMessage>, reverse: Bool) {
        self.messagesCollection = messagesCollection
        self.reverse = reverse
        Log.add(info: "Reverse Preference: \(reverse)")
        messagesToken?.invalidate()
        messagesToken = messagesCollection.observe { [weak self] collection, _, error in
            Log.add(info: "Message Changed ")
            
            guard let weakSelf = self else {
                return
            }
            
            if let error {
                weakSelf.dataSourceObserver?.didUpdateDataSource(hasNewMessage: false, error: error)
                return
            }
            
            let messages = weakSelf.messagesCollection.allObjects()
            let sortedMessages = messages.sorted { $0.createdAt < $1.createdAt}
            
            let hash = sortedMessages.last?.model.hashValue ?? 0
            
            let newMessageArrived = weakSelf.lastMessageHash != hash
            weakSelf.lastMessageHash = hash
            
            let logMessages = messagesCollection.allObjects().map { "\($0.data?["text"] as? String ?? "") | \($0.messageId)" }
            print("Message Collection -----> \(logMessages)")
            
            if !weakSelf.isFirstPageLoaded {
                let latestMessage = messagesCollection.allObjects().first
                latestMessage?.markRead()
                
                weakSelf.isFirstPageLoaded = true
            }
            
            weakSelf.dataSourceObserver?.didUpdateDataSource(hasNewMessage: newMessageArrived, error: nil)
        }
    }

    func numberOfMessages() -> Int {
        return Int(messagesCollection.count())
    }

    func message(for indexPath: IndexPath) -> AmityMessage? {
        let row: Int
        if reverse {
            row = numberOfMessages() - indexPath.row - 1
        } else {
            row = indexPath.row
        }
        
        guard row >= 0 else {
            return nil
        }
        
        return messagesCollection.object(at: row)
    }

    func loadNext() {
        if messagesCollection.hasNext {
            messagesCollection.nextPage()
        }
    }
    
    func loadPrevious() {
        if messagesCollection.hasPrevious {
            messagesCollection.previousPage()
        }
    }
}
