//
//  ChannelIdListInputView.swift
//  SampleApp
//
//  Created by Nishan on 7/12/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import SwiftUI

struct ChannelIdListInputView: View {
    
    @Environment(\.dismiss) var dismissScreen
    
    struct Input: Identifiable {
        let id: UUID = UUID()
        var text: String = ""
    }
    
    @State private var allInputs: [Input] = []
    var queryAction: (([String]) -> Void)?
    
    var body: some View {
        List {
            ForEach($allInputs) { item in
                TextField("Channel Id", text: item.text)
            }
            .onDelete(perform: { indexSet in
                delete(at: indexSet)
            })
            
            Button(action: {
                let allChannelIds = allInputs.compactMap {
                    let id = $0.text.trimmingCharacters(in: .whitespacesAndNewlines)
                    if !id.isEmpty {
                        return id
                    } else {
                        return nil
                    }
                }
                
                queryAction?(allChannelIds)
                
                // Pop the screen
                dismissScreen()
                
            }, label: {
                HStack {
                    Spacer()
                    Text("Query Channels")
                    Spacer()
                }
            })
            .disabled(allInputs.isEmpty)
            .accessibilityIdentifier("query-channel-button")
        }
        .navigationTitle("Channel Id")
        .navigationBarTitleDisplayMode(.inline)
        .toolbar(content: {
            Button(action: {
                allInputs.append(Input(text: ""))
            }, label: {
                Image(systemName: "plus")
            })
            .accessibilityIdentifier("add-new-channel-id-button")
        })
    }
    
    func delete(at offsets: IndexSet) {
        allInputs.remove(atOffsets: offsets)
    }
}

#Preview {
    NavigationView(content: {
        ChannelIdListInputView()
    })
    
}
