//
//  SingleCommunityView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 6/23/22.
//  Copyright © 2022 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

/// Using LiveObject from another Observable Object in SwiftUI View.
struct SingleCommunityView: View {
    
    @State var commId: String = ""
    @State var commDetails: String = "-"
    @State var token: AmityNotificationToken?
    
    @StateObject var viewModel = SingleCommunityViewModel()
        
    var body: some View {
        
        VStack {
            TextField("Enter Community Id", text: $commId)
                .frame(height: 50)
                .textFieldStyle(RoundedBorderTextFieldStyle())
            
            Button {
                guard !commId.isEmpty else {
                    commDetails = "Please enter community id first."
                    return
                }
                
                viewModel.setCommunityId(id: commId)
            } label: {
                Text("Fetch")
                    .padding()
                    .frame(width: 150, height: 45)
                    .foregroundColor(Color.white)
                    .background(Color.blue)
                    .clipped()
                    .cornerRadius(8)
            }
            .padding(.top, 24)
                        
            if !viewModel.communityId.isEmpty {
                LiveCommunityView(liveObject: viewModel.getCommunityLiveObject())
            }

            Spacer()
        }
        .padding()
    }

}

struct SingleCommunityView_Previews: PreviewProvider {
    static var previews: some View {
        SingleCommunityView()
    }
}

class SingleCommunityViewModel: ObservableObject {
    
    var commRepo = AmityCommunityRepository(client: AmityManager.shared.client!)

    @Published var communityId: String = ""
    
    private var liveObject: AmityObject<AmityCommunity>?
    
    func setCommunityId(id: String) {
        self.communityId = id
    }

    func getCommunityLiveObject() -> AmityObject<AmityCommunity> {
        let commObject = commRepo.getCommunity(withId: communityId)
        self.liveObject = commObject
        return commObject
    }
}

struct LiveCommunityView: View {
    
    @ObservedObject var liveObject: AmityObject<AmityCommunity>
    
    var body: some View {
        VStack(alignment: .leading) {
            Text(liveObject.snapshot?.modelDescription ?? "-")
            Text("Loading Status: \(liveObject.loadingStatus.description)")
            Text("Data Status: \(liveObject.dataStatus.description)")
        }
    }
}
