//
//  CustomPostView.swift
//  SampleApp
//
//  Created by Nishan on 8/2/2567 BE.
//  Copyright © 2567 BE David Zhang. All rights reserved.
//

import SwiftUI

struct CustomPostView: View {
    
    @State var dataType: String = "amity.custom"
    
    @State var key: String = ""
    @State var textValue: String = ""
    @State var arrayValue: String = ""
    
    @State var customObjectKey: String = ""
    @State var customObjectValue: String = ""
    
    @ObservedObject var viewModel = CustomPostViewModel()
    
    @Environment(\.dismiss) var dismiss
    
    var sendButtonAction: (([String: Any]) -> Void)?

    
    let note: String = """
    Note:

    Custom post accepts a json object as an input from user. So use this form to create the json object.

    1. Add key value pair. A single key can have a single type of value. Value types can be one of the text, array of strings or custom object.

    2. You can add as many key value pair as you like. Fill up the input & tap on create

    3. Your custom json structure will be shown at the bottom of the screen.

    4. Tap on send button to create a post.
    """
    
    
    var body: some View {
        Form {
            
            Section(header: Text("Data Type")) {
                TextField("Data Type", text: $dataType)
            }
            .disabled(true)
            
            Section(header: Text("Enter Key")) {
                TextField("Key", text: $key)
                    .textInputAutocapitalization(.never)
            }
            
            Section(header: Text("Enter Value"), footer: Text("For array use comma (,) separated string")) {
                                
                TextField("Single Value", text: $textValue)
                    .textInputAutocapitalization(.never)
                TextField("Array", text: $arrayValue)
                    .textInputAutocapitalization(.never)
                Text("Or set custom object as value")
                    .font(.system(size: 14))
                
                TextField("Key", text: $customObjectKey)
                    .textInputAutocapitalization(.never)
                TextField("Value", text: $customObjectValue)
                    .textInputAutocapitalization(.never)
                
                Button(action: {
                    guard !key.isEmpty else { return }
                    
                    if !textValue.isEmpty {
                        viewModel.userInput[key] = textValue
                    } else if !arrayValue.isEmpty {
                        let arr = arrayValue.components(separatedBy: ",").map {  $0.trimmingCharacters(in: .whitespacesAndNewlines) }
                        viewModel.userInput[key] = arr
                    } else if !customObjectKey.isEmpty {
                        let newPair = [customObjectKey: customObjectValue]
                        viewModel.userInput[key] = newPair
                    }
                    
                    key = ""
                    textValue = ""
                    arrayValue = ""
                    customObjectKey = ""
                    customObjectValue = ""
                    
                }, label: {
                    HStack {
                        Spacer()
                        Text("Add")
                        Spacer()
                    }
                })
            }
            
            Section {
                Text("Your Data: \n\(viewModel.getUserInputDescription())")
            }
            
            Text(note)
                .font(.system(size: 14))
                .foregroundColor(Color(UIColor.darkGray))
                .padding(.vertical)
            
            
            Section {
                Button(action: {
                    sendButtonAction?(viewModel.userInput)
                    
                    dismiss()
                }, label: {
                    HStack {
                        Spacer()
                        Text("Set Custom Post Data")
                        Spacer()
                    }
                })
            }
            
            
        }
        
    }
}

struct CustomPostView_Previews: PreviewProvider {
    static var previews: some View {
        CustomPostView(key: "")
    }
}

class CustomPostViewModel: ObservableObject {
    
    @Published var userInput = [String: Any]()
    
    func getUserInputDescription() -> String {
        
        let serializedData = try? JSONSerialization.data(withJSONObject: userInput, options: .prettyPrinted)
        
        if let data = serializedData, let prettyPrintedData = String(data: data, encoding: .utf8) {
            return prettyPrintedData
        } else {
            return userInput.debugDescription
        }
    }
}
