//
//  StoryRTEView.swift
//  SampleApp
//
//  Created by Manuchet Rungraksa on 17/1/2567 BE.
//  Copyright © 2567 BE David Zhang. All rights reserved.
//

import Foundation
import SwiftUI
import AmitySDK

struct StoryRTEView: View {
    
    @StateObject var viewModel: StoryRTEViewModel
    @State var showAlert: Bool = false
    @State var alertText = ""
    
    var body: some View {
        VStack {
            Text(viewModel.displayText)
            Button("Subscribe Story Event") {
                viewModel.story.subscribeEvent(.story) { success, error in
                    showAlert = true
                    if let error {
                        alertText = "Failed to unsubscribe \(error.localizedDescription)"
                    }
                    alertText = "Subscribe to story"
                }
            }
            Button("Unsubscribe Story Event") {
                viewModel.story.unsubscribeEvent(.story) { success, error in
                    showAlert = true
                    if let error {
                        alertText = "Failed to unsubscribe \(error.localizedDescription)"
                    }
                    alertText = "Unsubsribe to story"
                }
            }
        }
        .alert(isPresented: $showAlert) {
            Alert(
                title: Text(alertText),
                dismissButton: .default(Text("OK")) {
                }
            )
            
        }
    }
}

class StoryRTEViewModel: ObservableObject {
    
    var token: AmityNotificationToken?
    let storyRepo = AmityStoryRepository(client: AmityManager.shared.client!)
    let story: AmityStory
    
    @Published var displayText: String = ""
        
    init(story: AmityStory, type: AmityReactionReferenceType = .message) {
        self.story = story
        self.token = storyRepo.getStory(storyId: story.storyId).observe({ [weak self] liveObject, error in
            guard let story = liveObject.snapshot else { return }
            let storyModel = StoryModel(story: story)
            self?.displayText = ""
            for data in storyModel.getPreviewData() {
                self?.displayText += "\(data.key): \(data.value)\n"
            }
        })
    }
    
}
