//
//  AppDelegate.swift
//  SampleApp
//
//  Created by Federico Zanetello on 5/27/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit
import UserNotifications
import SwiftUI
import AmitySDK
#if DISTRIBUTION
import Firebase
#endif

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate, UNUserNotificationCenterDelegate {
    
    var window: UIWindow?
    
    func application(_ application: UIApplication, willFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]? = nil) -> Bool {
        
        AmityLog.logLevel = .all
        
        UNUserNotificationCenter.current().delegate = self
        UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .sound, .badge]) { granted, error in
            if let error = error {
                Log.add(info: "🛑 Failed to authorize: \(error.localizedDescription)")
            } else {
                DispatchQueue.main.async {
                    UIApplication.shared.registerForRemoteNotifications()
                }
            }
        }
        
        UIScrollView.appearance().keyboardDismissMode = .onDrag
        
        let navAppearance = UINavigationBarAppearance()
        navAppearance.configureWithOpaqueBackground()
        
        UINavigationBar.appearance().standardAppearance = navAppearance
        UINavigationBar.appearance().compactAppearance = navAppearance
        UINavigationBar.appearance().scrollEdgeAppearance = navAppearance
        
        let controller = UIHostingController(rootView: MainContainerView())
        self.window = UIWindow(frame: UIScreen.main.bounds)
        self.window?.rootViewController = controller
        self.window?.makeKeyAndVisible()
        
#if DISTRIBUTION
        FirebaseApp.configure()
#endif
        return true
    }
    
    func applicationDidBecomeActive(_ application: UIApplication) {
        // Removes the Push Notification Badge number on the app icon.
        UIApplication.shared.applicationIconBadgeNumber = 0
    }
    
    // MARK: Push Notifications Token Management
    
    func application( _ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {        
        let tokenParts: [String] = deviceToken.map { data in String(format: "%02.2hhx", data) }
        let token: String = tokenParts.joined()
        
        UserDefaults.standard.deviceToken = token
        UserDefaults.standard.isRegisterdForPushNotification = true
    }
    
    func application(_ application: UIApplication, didFailToRegisterForRemoteNotificationsWithError error: Error) {
        Log.add(info: "🛑 Failed to register: \(error.localizedDescription)")
    }
    
    // MARK: UNUserNotificationCenterDelegate
    
    func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        
        let userInfo = notification.request.content.userInfo
        AmityManager.shared.appendNotificationPayload(userInfo)
        
        completionHandler([.banner, .sound])
        
    }
    
    func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        let userInfo = response.notification.request.content.userInfo
        NotificationCenter.default.post(name: NSNotification.Name.notificationClicked, object: nil, userInfo: userInfo)
    }
    
}

